package main

import (
	"fmt"
	"os"

	"github.com/spf13/cobra"
	"github.com/spf13/viper"
	"go.mills.io/saltyim"
)

var setavatarCmd = &cobra.Command{
	Use:   "setavatar <file>",
	Short: "Creates or updates your avatar with a Salty Broker",
	Long: `This command creates or updates your avatar with a regisetered account
on a Salty Broker (an instance of saltyd).

NOTE: This is only spported on a Salty Broker.`,
	Args: cobra.ExactArgs(1),
	Run: func(cmd *cobra.Command, args []string) {
		user := viper.GetString("user")
		identity := viper.GetString("identity")

		var profiles []profile
		viper.UnmarshalKey("profiles", &profiles)
		for _, p := range profiles {
			if user == p.User {
				identity = p.Identity
			}
		}

		me, err := saltyim.ParseAddr(user)
		if err != nil {
			fmt.Fprintf(os.Stderr, "error parsing addr: %s\n", err)
		}
		// XXX: What if me.IsZero()

		setavatar(me, identity, args[0])
	},
}

func init() {
	rootCmd.AddCommand(setavatarCmd)
}

func setavatar(me *saltyim.Addr, identity, fn string) {
	cli, err := saltyim.NewClient(me, saltyim.WithClientIdentity(saltyim.WithIdentityPath(identity)))
	if err != nil {
		fmt.Fprintf(os.Stderr, "error initializing client: %s\n", err)
		os.Exit(2)
	}

	data, err := os.ReadFile(fn)
	if err != nil {
		fmt.Fprintf(os.Stderr, "error reading avatar file %s: %s", fn, err)
		os.Exit(2)
	}

	if err := cli.SetAvatar(data); err != nil {
		fmt.Fprintf(os.Stderr, "error updating avatar: %s\n", err)
		os.Exit(2)
	}

	fmt.Println("Success! 🥳")
}
